// Copyright (c) FIRST and other WPILib contributors.
// Open Source Software; you can modify and/or share it under the terms of
// the WPILib BSD license file in the root directory of this project.

package frc.robot;

import edu.wpi.first.wpilibj.shuffleboard.BuiltInWidgets;
import edu.wpi.first.wpilibj.shuffleboard.Shuffleboard;
import edu.wpi.first.wpilibj.shuffleboard.ShuffleboardTab;
import edu.wpi.first.wpilibj2.command.Command;
import edu.wpi.first.wpilibj2.command.Commands;
import edu.wpi.first.wpilibj2.command.button.CommandXboxController;
import frc.robot.constants.OIConstants;
import frc.robot.constants.ElevatorConstants.ElevatorPositions;
import frc.robot.subsystems.Elevator;

public class RobotContainer {
  private Elevator elevator;

  private CommandXboxController driver;

  public RobotContainer() {
    elevator = new Elevator();

    driver = new CommandXboxController(OIConstants.kDriverUSB);

    configureBindings();
    configureShuffleboard();
  }

  private void configureBindings() {
    elevator.setDefaultCommand(
      elevator.assistedManualControl(driver::getRightY)
    );

    driver.a().onTrue(elevator.goToPosition(ElevatorPositions.kBottom, 3));
    driver.b().onTrue(elevator.goToPosition(ElevatorPositions.kLowMid, 3));
    driver.x().onTrue(elevator.goToPosition(ElevatorPositions.kHighMid, 3));
    driver.y().onTrue(elevator.goToPosition(ElevatorPositions.kTop, 3));
  }

  private void configureShuffleboard() {
    ShuffleboardTab tab = Shuffleboard.getTab("MyTab");

    tab.addDouble("Elevator Encoder Distance", elevator::getEncoderDistance)
      .withPosition(0, 0)
      .withSize(2, 1)
      .withWidget(BuiltInWidgets.kTextView);

    tab.addDouble("Elevator Encoder Rate", elevator::getEncoderRate)
      .withPosition(2, 1)
      .withSize(2, 1)
      .withWidget(BuiltInWidgets.kTextView);

    tab.addDouble("Elevator Bottom Setpoint", ElevatorPositions.kBottom::getPositionValue)
      .withPosition(0, 1)
      .withSize(2, 1)
      .withWidget(BuiltInWidgets.kTextView);

    tab.addDouble("Elevator Low-Mid Setpoint", ElevatorPositions.kLowMid::getPositionValue)
      .withPosition(2, 1)
      .withSize(2, 1)
      .withWidget(BuiltInWidgets.kTextView);

    tab.addDouble("Elevator High-Mid Setpoint", ElevatorPositions.kHighMid::getPositionValue)
      .withPosition(0, 2)
      .withSize(2, 1)
      .withWidget(BuiltInWidgets.kTextView);

    tab.addDouble("Elevator Top Setpoint", ElevatorPositions.kTop::getPositionValue)
      .withPosition(2, 2)
      .withSize(2, 1)
      .withWidget(BuiltInWidgets.kTextView);
  }

  public Command getAutonomousCommand() {
    return Commands.print("No autonomous command configured");
  }
}
